@@ -1,17 +1,18 @@
-import numpy as np
-from matplotlib.colors import ListedColormap
-from matplotlib.colorbar import ColorbarBase, Colorbar
-from matplotlib.axes import SubplotBase
-from seaborn import heatmap
-from pandas import DataFrame
 from typing import Union, List, Tuple
 
+import numpy as np
+from matplotlib import colors
+from matplotlib.axes import SubplotBase
+from matplotlib.colorbar import ColorbarBase, Colorbar
+from matplotlib.colors import ListedColormap
+from pandas import DataFrame
+from seaborn import heatmap
+
 
 def sign_array(
         p_values: Union[List, np.ndarray],
         alpha: float = 0.05) -> np.ndarray:
-    """
-    Significance array
+    """Significance array.
 
     Converts an array with p values to a significance array where
     0 is False (not significant), 1 is True (significant),
@@ -34,7 +35,6 @@ def sign_array(
 
     Examples
     --------
-
     >>> p_values = np.array([[ 1.        ,  0.00119517,  0.00278329],
                              [ 0.00119517,  1.        ,  0.18672227],
                              [ 0.00278329,  0.18672227,  1.        ]])
@@ -43,11 +43,11 @@ def sign_array(
            [1, 1, 0],
            [1, 0, 1]])
     """
-
     p_values = np.array(p_values)
     p_values[p_values > alpha] = 0
     p_values[(p_values < alpha) & (p_values > 0)] = 1
     np.fill_diagonal(p_values, 1)
+
     return p_values
 
 
@@ -79,7 +79,6 @@ def sign_table(
 
     Examples
     --------
-
     >>> p_values = np.array([[-1.        ,  0.00119517,  0.00278329],
                       [ 0.00119517, -1.        ,  0.18672227],
                       [ 0.00278329,  0.18672227, -1.        ]])
@@ -87,13 +86,12 @@ def sign_table(
     array([['-', '**', '**'],
            ['**', '-', 'NS'],
            ['**', 'NS', '-']], dtype=object)
-
     """
     if not any([lower, upper]):
         raise ValueError("Either lower or upper triangle must be returned")
 
-    pv = DataFrame(p_values, copy=True)\
-        if not isinstance(p_values, DataFrame)\
+    pv = DataFrame(p_values, copy=True) \
+        if not isinstance(p_values, DataFrame) \
         else p_values.copy()
 
     ns = pv > 0.05
@@ -132,7 +130,8 @@ def sign_plot(
     x : Union[List, np.ndarray, DataFrame]
         If flat is False (default), x must be an array, any object exposing
         the array interface, containing p values. If flat is True, x must be
-        a sign_array (returned by `scikit_posthocs.sign_array` function)
+        a sign_array (returned by :py:meth:`scikit_posthocs.sign_array`
+        function).
 
     g : Union[List, np.ndarray]
         An array, any object exposing the array interface, containing
@@ -190,7 +189,6 @@ def sign_plot(
                       [ 1, 0, 1]])
     >>> ph.sign_plot(x, flat = True)
     """
-
     for key in ['cbar', 'vmin', 'vmax', 'center']:
         if key in kwargs:
             del kwargs[key]
@@ -244,10 +242,9 @@ def sign_plot(
             hax.set_ylabel('')
 
         cbar_ax = hax.figure.add_axes(cbar_ax_bbox or [0.95, 0.35, 0.04, 0.3])
-        cbar = ColorbarBase(cbar_ax, cmap=ListedColormap(cmap[2:] + [cmap[1]]),
+        cbar = ColorbarBase(cbar_ax, cmap=(ListedColormap(cmap[2:] + [cmap[1]])), norm=colors.NoNorm(),
                             boundaries=[0, 1, 2, 3, 4])
-        cbar.set_ticks(np.linspace(0.5, 3.5, 4))
-        cbar.set_ticklabels(['p < 0.001', 'p < 0.01', 'p < 0.05', 'NS'])
+        cbar.set_ticks(list(np.linspace(0, 3, 4)), labels=['p < 0.001', 'p < 0.01', 'p < 0.05', 'NS'])
 
         cbar.outline.set_linewidth(1)
         cbar.outline.set_edgecolor('0.5')
