@@ -1,22 +1,25 @@
-# -*- coding: utf-8 -*-
-
+import itertools as it
+from typing import Tuple, Union
 import numpy as np
 import scipy.stats as ss
-import itertools as it
 from statsmodels.sandbox.stats.multicomp import multipletests
 from statsmodels.stats.multicomp import pairwise_tukeyhsd
 from statsmodels.stats.libqsturng import psturng
 from pandas import DataFrame
 
 
-def __convert_to_df(a, val_col: str = 'vals', group_col: str = 'groups',
-                    val_id: int = None, group_id: int = None) -> DataFrame:
+def __convert_to_df(
+        a: Union[list, np.ndarray, DataFrame],
+        val_col: str = 'vals',
+        group_col: str = 'groups',
+        val_id: int = None,
+        group_id: int = None) -> Tuple[DataFrame, str, str]:
     '''Hidden helper method to create a DataFrame with input data for further
     processing.
 
     Parameters
     ----------
-    a : array_like or pandas DataFrame object
+    a : Union[list, np.ndarray, DataFrame]
         An array, any object exposing array interface or a pandas DataFrame.
         Array must be two-dimensional. Second dimension may vary, i.e. groups
         may have different lengths.
@@ -43,24 +46,20 @@ def __convert_to_df(a, val_col: str = 'vals', group_col: str = 'groups',
 
     Returns
     -------
-    x : pandas DataFrame
-        DataFrame with input data, `val_col` column contains numerical values and
-        `group_col` column contains categorical values.
-
-    val_col : str
-        Name of a DataFrame column that contains dependent variable values (test
-        or response variable).
-
-    group_col : str
-        Name of a DataFrame column that contains independent variable values
-        (grouping or predictor variable).
+    Tuple[DataFrame, str, str]
+        Returns a tuple of DataFrame and two strings:
+        - DataFrame with input data, `val_col` column contains numerical values
+          and `group_col` column contains categorical values.
+        - Name of a DataFrame column that contains dependent variable values
+          (test or response variable).
+        - Name of a DataFrame column that contains independent variable values
+          (grouping or predictor variable).
 
     Notes
     -----
     Inferrence algorithm for determining `val_id` and `group_id` args is rather
     simple, so it is better to specify them explicitly to prevent errors.
     '''
-
     if not group_col:
         group_col = 'groups'
     if not val_col:
@@ -83,7 +82,7 @@ def __convert_to_df(a, val_col: str = 'vals', group_col: str = 'groups',
 
         # cols ids not defined
         # trying to infer
-        if not(all([val_id, group_id])):
+        if not all([val_id, group_id]):
 
             if np.argmax(a.shape):
                 a = a.T
@@ -106,7 +105,12 @@ def __convert_to_df(a, val_col: str = 'vals', group_col: str = 'groups',
         return DataFrame(a, columns=cols_vals), val_col, group_col
 
 
-def __convert_to_block_df(a, y_col=None, group_col=None, block_col=None, melted=False):
+def __convert_to_block_df(
+        a,
+        y_col: str = None,
+        group_col: str = None,
+        block_col: str = None,
+        melted: bool = False) -> DataFrame:
     # TODO: refactor conversion of block data to DataFrame
     if melted and not all([i is not None for i in [block_col, group_col, y_col]]):
         raise ValueError('`block_col`, `group_col`, `y_col` should be explicitly specified if using melted data')
@@ -147,9 +151,14 @@ def __convert_to_block_df(a, y_col=None, group_col=None, block_col=None, melted=
     return x, y_col, group_col, block_col
 
 
-def posthoc_conover(a, val_col=None, group_col=None, p_adjust=None, sort=True):
+def posthoc_conover(
+        a: Union[list, np.ndarray, DataFrame],
+        val_col: str = None,
+        group_col: str = None,
+        p_adjust: str = None,
+        sort: bool = True) -> DataFrame:
     '''Post hoc pairwise test for multiple comparisons of mean rank sums
-    (Conover's test). May be used after Kruskal-Wallis one-way analysis of
+    (Conover´s test). May be used after Kruskal-Wallis one-way analysis of
     variance by ranks to do pairwise comparisons [1]_.
 
     Parameters
@@ -189,7 +198,7 @@ def posthoc_conover(a, val_col=None, group_col=None, p_adjust=None, sort=True):
 
     Returns
     -------
-    result : pandas DataFrame
+    result : pandas.DataFrame
         P values.
 
     Notes
@@ -198,16 +207,14 @@ def posthoc_conover(a, val_col=None, group_col=None, p_adjust=None, sort=True):
 
     References
     ----------
-    .. [1] W. J. Conover and R. L. Iman (1979), On multiple-comparisons procedures,
-        Tech. Rep. LA-7677-MS, Los Alamos Scientific Laboratory.
+    .. [1] W. J. Conover and R. L. Iman (1979), On multiple-comparisons
+        procedures, Tech. Rep. LA-7677-MS, Los Alamos Scientific Laboratory.
 
     Examples
     --------
-
     >>> x = [[1,2,3,5,1], [12,31,54, np.nan], [10,12,6,74,11]]
     >>> sp.posthoc_conover(x, p_adjust = 'holm')
     '''
-
     def compare_conover(i, j):
         diff = np.abs(x_ranks_avg.loc[i] - x_ranks_avg.loc[j])
         B = (1. / x_lens.loc[i] + 1. / x_lens.loc[j])
@@ -260,7 +267,12 @@ def posthoc_conover(a, val_col=None, group_col=None, p_adjust=None, sort=True):
     return DataFrame(vs, index=x_groups_unique, columns=x_groups_unique)
 
 
-def posthoc_dunn(a, val_col=None, group_col=None, p_adjust=None, sort=True):
+def posthoc_dunn(
+        a: Union[list, np.ndarray, DataFrame],
+        val_col: str = None,
+        group_col: str = None,
+        p_adjust: str = None,
+        sort: bool = True) -> DataFrame:
     '''Post hoc pairwise test for multiple comparisons of mean rank sums
     (Dunn's test). May be used after Kruskal-Wallis one-way analysis of
     variance by ranks to do pairwise comparisons [1]_, [2]_.
@@ -302,7 +314,7 @@ def posthoc_dunn(a, val_col=None, group_col=None, p_adjust=None, sort=True):
 
     Returns
     -------
-    result : pandas DataFrame
+    result : pandas.DataFrame
         P values.
 
     Notes
@@ -364,7 +376,12 @@ def posthoc_dunn(a, val_col=None, group_col=None, p_adjust=None, sort=True):
     return DataFrame(vs, index=x_groups_unique, columns=x_groups_unique)
 
 
-def posthoc_nemenyi(a, val_col=None, group_col=None,  dist='chi', sort=True):
+def posthoc_nemenyi(
+        a: Union[list, np.ndarray, DataFrame],
+        val_col: str = None,
+        group_col: str = None,
+        dist: str = 'chi',
+        sort: bool = True) -> DataFrame:
     '''Post hoc pairwise test for multiple comparisons of mean rank sums
     (Nemenyi's test). May be used after Kruskal-Wallis one-way analysis of
     variance by ranks to do pairwise comparisons [1]_.
@@ -396,7 +413,7 @@ def posthoc_nemenyi(a, val_col=None, group_col=None,  dist='chi', sort=True):
 
     Returns
     -------
-    result : pandas DataFrame
+    result : pandas.DataFrame
         P values.
 
     Notes
@@ -413,7 +430,6 @@ def posthoc_nemenyi(a, val_col=None, group_col=None,  dist='chi', sort=True):
     >>> x = [[1,2,3,5,1], [12,31,54, np.nan], [10,12,6,74,11]]
     >>> sp.posthoc_nemenyi(x)
     '''
-
     def compare_stats_chi(i, j):
         diff = np.abs(x_ranks_avg.loc[i] - x_ranks_avg.loc[j])
         A = n * (n + 1.) / 12.
@@ -469,7 +485,13 @@ def posthoc_nemenyi(a, val_col=None, group_col=None,  dist='chi', sort=True):
     return DataFrame(vs, index=x_groups_unique, columns=x_groups_unique)
 
 
-def posthoc_nemenyi_friedman(a, y_col=None, block_col=None, group_col=None, melted=False, sort=False):
+def posthoc_nemenyi_friedman(
+        a: Union[list, np.ndarray, DataFrame],
+        y_col: str = None,
+        block_col: str = None,
+        group_col: str = None,
+        melted: bool = False,
+        sort: bool = False) -> DataFrame:
     '''Calculate pairwise comparisons using Nemenyi post hoc test for
     unreplicated blocked data. This test is usually conducted post hoc if
     significant results of the Friedman's test are obtained. The statistics
@@ -514,7 +536,7 @@ def posthoc_nemenyi_friedman(a, y_col=None, block_col=None, group_col=None, melt
 
     Returns
     -------
-    result : pandas DataFrame
+    result : pandas.DataFrame
         P values.
 
     Notes
@@ -544,7 +566,6 @@ def posthoc_nemenyi_friedman(a, y_col=None, block_col=None, group_col=None, melt
     >>> x = np.array([[31,27,24],[31,28,31],[45,29,46],[21,18,48],[42,36,46],[32,17,40]])
     >>> sp.posthoc_nemenyi_friedman(x)
     '''
-
     def compare_stats(i, j):
         dif = np.abs(R[groups[i]] - R[groups[j]])
         qval = dif / np.sqrt(k * (k + 1.) / (6. * n))
@@ -577,7 +598,14 @@ def posthoc_nemenyi_friedman(a, y_col=None, block_col=None, group_col=None, melt
     return DataFrame(vs, index=groups, columns=groups)
 
 
-def posthoc_conover_friedman(a, y_col=None, block_col=None, group_col=None, melted=False, sort=False, p_adjust=None):
+def posthoc_conover_friedman(
+        a: Union[list, np.ndarray, DataFrame],
+        y_col: str = None,
+        block_col: str = None,
+        group_col: str = None,
+        melted: bool = False,
+        sort: bool = False,
+        p_adjust: str = None) -> DataFrame:
     '''Calculate pairwise comparisons using Conover post hoc test for unreplicated
     blocked data. This test is usually conducted post hoc after
     significant results of the Friedman test. The statistics refer to
@@ -636,7 +664,7 @@ def posthoc_conover_friedman(a, y_col=None, block_col=None, group_col=None, melt
 
     Returns
     -------
-    result : pandas DataFrame
+    result : pandas.DataFrame
         P values.
 
     Notes
@@ -662,7 +690,6 @@ def posthoc_conover_friedman(a, y_col=None, block_col=None, group_col=None, melt
     >>> x = np.array([[31,27,24],[31,28,31],[45,29,46],[21,18,48],[42,36,46],[32,17,40]])
     >>> sp.posthoc_conover_friedman(x)
     '''
-
     def compare_stats(i, j):
         dif = np.abs(R.loc[groups[i]] - R.loc[groups[j]])
         tval = dif / np.sqrt(A) / np.sqrt(B)
@@ -714,11 +741,15 @@ def posthoc_conover_friedman(a, y_col=None, block_col=None, group_col=None, melt
     return DataFrame(vs, index=groups, columns=groups)
 
 
-def posthoc_npm_test(a, val_col=None, group_col=None, sort=False, p_adjust=None):
-    '''Calculate pairwise comparisons using Nashimoto and Wright's all-pairs
+def posthoc_npm_test(
+        a: Union[list, np.ndarray, DataFrame],
+        val_col: str = None,
+        group_col: str = None,
+        sort: bool = False) -> DataFrame:
+    '''Calculate pairwise comparisons using Nashimoto and Wright´s all-pairs
     comparison procedure (NPM test) for simply ordered mean ranksums.
 
-    NPM test is basically an extension of Nemenyi's procedure for testing
+    NPM test is basically an extension of Nemenyi´s procedure for testing
     increasingly ordered alternatives [1]_.
 
     Parameters
@@ -756,7 +787,7 @@ def posthoc_npm_test(a, val_col=None, group_col=None, sort=False, p_adjust=None)
 
     Returns
     -------
-    result : pandas DataFrame
+    result : pandas.DataFrame
         P values.
 
     Notes
@@ -778,7 +809,6 @@ def posthoc_npm_test(a, val_col=None, group_col=None, sort=False, p_adjust=None)
                       [132,141,156,160,172]])
     >>> sp.posthoc_npm_test(x)
     '''
-
     x, _val_col, _group_col = __convert_to_df(a, val_col, group_col)
     x = x.sort_values(by=[_group_col], ascending=True) if sort else x
 
@@ -814,8 +844,15 @@ def posthoc_npm_test(a, val_col=None, group_col=None, sort=False, p_adjust=None)
     return DataFrame(p_values, index=groups, columns=groups)
 
 
-def posthoc_siegel_friedman(a, y_col=None, block_col=None, group_col=None, melted=False, sort=False, p_adjust=None):
-    '''Siegel and Castellan's All-Pairs Comparisons Test for Unreplicated Blocked
+def posthoc_siegel_friedman(
+        a: Union[list, np.ndarray, DataFrame],
+        y_col: str = None,
+        block_col: str = None,
+        group_col: str = None,
+        p_adjust: str = None,
+        melted: bool = False,
+        sort: bool = False) -> DataFrame:
+    '''Siegel and Castellan´s All-Pairs Comparisons Test for Unreplicated Blocked
     Data. See authors' paper for additional information [1]_.
 
     Parameters
@@ -869,7 +906,7 @@ def posthoc_siegel_friedman(a, y_col=None, block_col=None, group_col=None, melte
 
     Returns
     -------
-    result : pandas DataFrame
+    result : pandas.DataFrame
         P values.
 
     Notes
@@ -888,7 +925,6 @@ def posthoc_siegel_friedman(a, y_col=None, block_col=None, group_col=None, melte
     >>> x = np.array([[31,27,24],[31,28,31],[45,29,46],[21,18,48],[42,36,46],[32,17,40]])
     >>> sp.posthoc_siegel_friedman(x)
     '''
-
     def compare_stats(i, j):
         dif = np.abs(R[groups[i]] - R[groups[j]])
         zval = dif / np.sqrt(k * (k + 1.) / (6. * n))
@@ -925,8 +961,14 @@ def posthoc_siegel_friedman(a, y_col=None, block_col=None, group_col=None, melte
     return DataFrame(vs, index=groups, columns=groups)
 
 
-def posthoc_miller_friedman(a, y_col=None, block_col=None, group_col=None, melted=False, sort=False):
-    '''Miller's All-Pairs Comparisons Test for Unreplicated Blocked Data.
+def posthoc_miller_friedman(
+        a: Union[list, np.ndarray, DataFrame],
+        y_col: str = None,
+        block_col: str = None,
+        group_col: str = None,
+        melted: bool = False,
+        sort: bool = False) -> DataFrame:
+    '''Miller´s All-Pairs Comparisons Test for Unreplicated Blocked Data.
     The p-values are computed from the chi-square distribution [1]_, [2]_,
     [3]_.
 
@@ -968,7 +1010,8 @@ def posthoc_miller_friedman(a, y_col=None, block_col=None, group_col=None, melte
 
     Returns
     -------
-    Pandas DataFrame containing p values.
+    result : pandas.DataFrame
+        P values.
 
     Notes
     -----
@@ -992,7 +1035,6 @@ def posthoc_miller_friedman(a, y_col=None, block_col=None, group_col=None, melte
     >>> x = np.array([[31,27,24],[31,28,31],[45,29,46],[21,18,48],[42,36,46],[32,17,40]])
     >>> sp.posthoc_miller_friedman(x)
     '''
-
     def compare_stats(i, j):
         dif = np.abs(R[groups[i]] - R[groups[j]])
         qval = dif / np.sqrt(k * (k + 1.) / (6. * n))
@@ -1025,7 +1067,14 @@ def posthoc_miller_friedman(a, y_col=None, block_col=None, group_col=None, melte
     return DataFrame(vs, index=groups, columns=groups)
 
 
-def posthoc_durbin(a, y_col=None, block_col=None, group_col=None, melted=False, sort=False, p_adjust=None):
+def posthoc_durbin(
+        a: Union[list, np.ndarray, DataFrame],
+        y_col: str = None,
+        block_col: str = None,
+        group_col: str = None,
+        p_adjust: str = None,
+        melted: bool = False,
+        sort: bool = False) -> DataFrame:
     '''Pairwise post hoc test for multiple comparisons of rank sums according to
     Durbin and Conover for a two-way balanced incomplete block design (BIBD). See
     references for additional information [1]_, [2]_.
@@ -1082,7 +1131,8 @@ def posthoc_durbin(a, y_col=None, block_col=None, group_col=None, melted=False,
 
     Returns
     -------
-    Pandas DataFrame containing p values.
+    result : pandas.DataFrame
+        P values.
 
     References
     ----------
@@ -1096,7 +1146,6 @@ def posthoc_durbin(a, y_col=None, block_col=None, group_col=None, melted=False,
     >>> x = np.array([[31,27,24],[31,28,31],[45,29,46],[21,18,48],[42,36,46],[32,17,40]])
     >>> sp.posthoc_durbin(x)
     '''
-
     x, y_col, group_col, block_col = __convert_to_block_df(a, y_col, group_col, block_col, melted)
 
     def compare_stats(i, j):
@@ -1140,7 +1189,13 @@ def posthoc_durbin(a, y_col=None, block_col=None, group_col=None, melted=False,
     return DataFrame(vs, index=groups, columns=groups)
 
 
-def posthoc_anderson(a, val_col=None, group_col=None, midrank=True, sort=False, p_adjust=None):
+def posthoc_anderson(
+        a: Union[list, np.ndarray, DataFrame],
+        val_col: str = None,
+        group_col: str = None,
+        midrank: bool = True,
+        p_adjust: str = None,
+        sort: bool = False) -> DataFrame:
     '''Anderson-Darling Pairwise Test for k-samples. Tests the null hypothesis
     that k-samples are drawn from the same population without having to specify
     the distribution function of that population [1]_.
@@ -1184,7 +1239,7 @@ def posthoc_anderson(a, val_col=None, group_col=None, midrank=True, sort=False,
 
     Returns
     -------
-    result : pandas DataFrame
+    result : pandas.DataFrame
         P values.
 
     References
@@ -1197,7 +1252,6 @@ def posthoc_anderson(a, val_col=None, group_col=None, midrank=True, sort=False,
     >>> x = np.array([[2.9, 3.0, 2.5, 2.6, 3.2], [3.8, 2.7, 4.0, 2.4], [2.8, 3.4, 3.7, 2.2, 2.0]])
     >>> sp.posthoc_anderson(x)
     '''
-
     x, _val_col, _group_col = __convert_to_df(a, val_col, group_col)
     x = x.sort_values(by=[_group_col], ascending=True) if sort else x
 
@@ -1221,7 +1275,15 @@ def posthoc_anderson(a, val_col=None, group_col=None, midrank=True, sort=False,
     return DataFrame(vs, index=groups, columns=groups)
 
 
-def posthoc_quade(a, y_col=None, block_col=None, group_col=None, dist='t', melted=False, sort=False, p_adjust=None):
+def posthoc_quade(
+        a: Union[list, np.ndarray, DataFrame],
+        y_col: str = None,
+        block_col: str = None,
+        group_col: str = None,
+        dist: str = 't',
+        p_adjust: str = None,
+        melted: bool = False,
+        sort: bool = False) -> DataFrame:
     '''Calculate pairwise comparisons using Quade's post hoc test for
     unreplicated blocked data. This test is usually conducted if significant
     results were obtained by the omnibus test [1]_, [2]_, [3]_.
@@ -1283,7 +1345,8 @@ def posthoc_quade(a, y_col=None, block_col=None, group_col=None, dist='t', melte
 
     Returns
     -------
-    Pandas DataFrame containing p values.
+    result : pandas.DataFrame
+        P values.
 
     References
     ----------
@@ -1303,7 +1366,6 @@ def posthoc_quade(a, y_col=None, block_col=None, group_col=None, dist='t', melte
     >>> x = np.array([[31,27,24],[31,28,31],[45,29,46],[21,18,48],[42,36,46],[32,17,40]])
     >>> sp.posthoc_quade(x)
     '''
-
     def compare_stats_t(i, j):
         dif = np.abs(S[groups[i]] - S[groups[j]])
         tval = dif / denom
@@ -1365,7 +1427,12 @@ def posthoc_quade(a, y_col=None, block_col=None, group_col=None, dist='t', melte
     return DataFrame(vs, index=groups, columns=groups)
 
 
-def posthoc_vanwaerden(a, val_col=None, group_col=None, sort=False, p_adjust=None):
+def posthoc_vanwaerden(
+        a: Union[list, np.ndarray, DataFrame],
+        val_col: str = None,
+        group_col: str = None,
+        sort: bool = False,
+        p_adjust: str = None) -> DataFrame:
     '''Van der Waerden's test for pairwise multiple comparisons between group
     levels. See references for additional information [1]_, [2]_.
 
@@ -1405,7 +1472,7 @@ def posthoc_vanwaerden(a, val_col=None, group_col=None, sort=False, p_adjust=Non
 
     Returns
     -------
-    result : pandas DataFrame
+    result : pandas.DataFrame
         P values.
 
     Notes
@@ -1432,7 +1499,6 @@ def posthoc_vanwaerden(a, val_col=None, group_col=None, sort=False, p_adjust=Non
     >>> x = np.array([[10,'a'], [59,'a'], [76,'b'], [10, 'b']])
     >>> sp.posthoc_vanwaerden(x, val_col = 0, group_col = 1)
     '''
-
     x, _val_col, _group_col = __convert_to_df(a, val_col, group_col)
     x = x.sort_values(by=[_group_col], ascending=True) if sort else x
 
@@ -1473,7 +1539,14 @@ def posthoc_vanwaerden(a, val_col=None, group_col=None, sort=False, p_adjust=Non
     return DataFrame(vs, index=groups, columns=groups)
 
 
-def posthoc_ttest(a, val_col=None, group_col=None, pool_sd=False, equal_var=True, p_adjust=None, sort=False):
+def posthoc_ttest(
+        a: Union[list, np.ndarray, DataFrame],
+        val_col: str = None,
+        group_col: str = None,
+        pool_sd: bool = False,
+        equal_var: bool = True,
+        p_adjust: str = None,
+        sort: bool = False) -> DataFrame:
     '''Pairwise T test for multiple comparisons of independent groups. May be
     used after a parametric ANOVA to do pairwise comparisons.
 
@@ -1526,8 +1599,8 @@ def posthoc_ttest(a, val_col=None, group_col=None, pool_sd=False, equal_var=True
 
     Returns
     -------
-    Numpy ndarray or pandas DataFrame of p values depending on input
-    data type.
+    result : pandas.DataFrame
+        P values.
 
     References
     ----------
@@ -1585,7 +1658,10 @@ def posthoc_ttest(a, val_col=None, group_col=None, pool_sd=False, equal_var=True
     return DataFrame(vs, index=groups, columns=groups)
 
 
-def posthoc_tukey_hsd(x, g, alpha=0.05):
+def posthoc_tukey_hsd(
+        x: Union[list, np.ndarray, DataFrame],
+        g: str,
+        alpha: float = 0.05) -> DataFrame:
     '''Pairwise comparisons with TukeyHSD confidence intervals. This is a
     convenience function to make statsmodels `pairwise_tukeyhsd` method more
     applicable for further use.
@@ -1608,7 +1684,7 @@ def posthoc_tukey_hsd(x, g, alpha=0.05):
 
     Returns
     -------
-    result : pandas DataFrame
+    result : pandas.DataFrame
         DataFrame with 0, 1, and -1 values, where 0 is False (not significant),
         1 is True (significant), and -1 is for diagonal elements.
 
@@ -1618,7 +1694,7 @@ def posthoc_tukey_hsd(x, g, alpha=0.05):
     >>> g = [['a'] * 5, ['b'] * 5, ['c'] * 5]
     >>> sp.posthoc_tukey_hsd(np.concatenate(x), np.concatenate(g))
     '''
-    result = pairwise_tukeyhsd(x, g, alpha=0.05)
+    result = pairwise_tukeyhsd(x, g, alpha=alpha)
     groups = np.array(result.groupsunique, dtype=str)
     groups_len = len(groups)
 
@@ -1638,7 +1714,14 @@ def posthoc_tukey_hsd(x, g, alpha=0.05):
     return DataFrame(vsu, index=groups, columns=groups)
 
 
-def posthoc_mannwhitney(a, val_col=None, group_col=None, use_continuity=True, alternative='two-sided', p_adjust=None, sort=True):
+def posthoc_mannwhitney(
+        a: Union[list, np.ndarray, DataFrame],
+        val_col: str = None,
+        group_col: str = None,
+        use_continuity: bool = True,
+        alternative: str = 'two-sided',
+        p_adjust: str = None,
+        sort: bool = True) -> DataFrame:
     '''Pairwise comparisons with Mann-Whitney rank test.
 
     Parameters
@@ -1687,7 +1770,7 @@ def posthoc_mannwhitney(a, val_col=None, group_col=None, use_continuity=True, al
 
     Returns
     -------
-    result : pandas DataFrame
+    result : pandas.DataFrame
         P values.
 
     Notes
@@ -1699,7 +1782,6 @@ def posthoc_mannwhitney(a, val_col=None, group_col=None, use_continuity=True, al
     >>> x = [[1,2,3,4,5], [35,31,75,40,21], [10,6,9,6,1]]
     >>> sp.posthoc_mannwhitney(x, p_adjust = 'holm')
     '''
-
     x, _val_col, _group_col = __convert_to_df(a, val_col, group_col)
     x = x.sort_values(by=[_group_col, _val_col], ascending=True) if sort else x
 
@@ -1728,9 +1810,18 @@ def posthoc_mannwhitney(a, val_col=None, group_col=None, use_continuity=True, al
     return DataFrame(vs, index=groups, columns=groups)
 
 
-def posthoc_wilcoxon(a, val_col=None, group_col=None, zero_method='wilcox', correction=False, p_adjust=None, sort=False):
-    '''Pairwise comparisons with Wilcoxon signed-rank test. It is a non-parametric
-    version of the paired T-test for use with non-parametric ANOVA.
+def posthoc_wilcoxon(
+        a: Union[list, np.ndarray, DataFrame],
+        val_col: str = None,
+        group_col: str = None,
+        zero_method: str = 'wilcox',
+        correction: bool = False,
+        p_adjust: str = None, 
+        sort: bool = False) -> DataFrame:
+    '''Pairwise comparisons with Wilcoxon signed-rank test.
+
+    It is a non-parametric version of the paired T-test for use with
+    non-parametric ANOVA.
 
     Parameters
     ----------
@@ -1768,9 +1859,9 @@ def posthoc_wilcoxon(a, val_col=None, group_col=None, zero_method='wilcox', corr
         'sidak' : one-step correction
         'holm-sidak' : step-down method using Sidak adjustments
         'holm' : step-down method using Bonferroni adjustments
-        'simes-hochberg' : step-up method  (independent)
+        'simes-hochberg' : step-up method (independent)
         'hommel' : closed method based on Simes tests (non-negative)
-        'fdr_bh' : Benjamini/Hochberg  (non-negative)
+        'fdr_bh' : Benjamini/Hochberg (non-negative)
         'fdr_by' : Benjamini/Yekutieli (negative)
         'fdr_tsbh' : two stage fdr correction (non-negative)
         'fdr_tsbky' : two stage fdr correction (non-negative)
@@ -1781,7 +1872,7 @@ def posthoc_wilcoxon(a, val_col=None, group_col=None, zero_method='wilcox', corr
 
     Returns
     -------
-    result : pandas DataFrame
+    result : pandas.DataFrame
         P values.
 
     Notes
@@ -1793,7 +1884,6 @@ def posthoc_wilcoxon(a, val_col=None, group_col=None, zero_method='wilcox', corr
     >>> x = [[1,2,3,4,5], [35,31,75,40,21], [10,6,9,6,1]]
     >>> sp.posthoc_wilcoxon(x)
     '''
-
     x, _val_col, _group_col = __convert_to_df(a, val_col, group_col)
     x = x.sort_values(by=[_group_col, _val_col], ascending=True) if sort else x
 
@@ -1821,7 +1911,11 @@ def posthoc_wilcoxon(a, val_col=None, group_col=None, zero_method='wilcox', corr
     return DataFrame(vs, index=groups, columns=groups)
 
 
-def posthoc_scheffe(a, val_col=None, group_col=None, sort=False, p_adjust=None):
+def posthoc_scheffe(
+        a: Union[list, np.ndarray, DataFrame],
+        val_col: str = None,
+        group_col: str = None,
+        sort: bool = False) -> DataFrame:
     '''Scheffe's all-pairs comparisons test for normally distributed data with equal
     group variances. For all-pairs comparisons in an one-factorial layout with
     normally distributed residuals and equal variances Scheffe's test can be
@@ -1831,7 +1925,7 @@ def posthoc_scheffe(a, val_col=None, group_col=None, sort=False, p_adjust=None):
 
     Parameters
     ----------
-    a : array_like or pandas DataFrame object
+    a : Union[list, np.ndarray, DataFrame]
         An array, any object exposing the array interface or a pandas
         DataFrame.
 
@@ -1850,7 +1944,7 @@ def posthoc_scheffe(a, val_col=None, group_col=None, sort=False, p_adjust=None):
 
     Returns
     -------
-    result : pandas DataFrame
+    result : pandas.DataFrame
         P values.
 
     Notes
@@ -1875,7 +1969,6 @@ def posthoc_scheffe(a, val_col=None, group_col=None, sort=False, p_adjust=None):
     >>> x = x.melt(var_name='groups', value_name='values')
     >>> sp.posthoc_scheffe(x, val_col='values', group_col='groups')
     '''
-
     x, _val_col, _group_col = __convert_to_df(a, val_col, group_col)
     x = x.sort_values(by=[_group_col], ascending=True) if sort else x
 
@@ -1909,7 +2002,12 @@ def posthoc_scheffe(a, val_col=None, group_col=None, sort=False, p_adjust=None):
     return DataFrame(p_values, index=groups, columns=groups)
 
 
-def posthoc_tamhane(a, val_col=None, group_col=None, welch=True, sort=False):
+def posthoc_tamhane(
+        a: Union[list, np.ndarray, DataFrame],
+        val_col: str = None,
+        group_col: str = None,
+        welch: bool = True,
+        sort: bool = False) -> DataFrame:
     '''Tamhane's T2 all-pairs comparison test for normally distributed data with
     unequal variances. Tamhane's T2 test can be performed for all-pairs
     comparisons in an one-factorial layout with normally distributed residuals
@@ -1919,7 +2017,7 @@ def posthoc_tamhane(a, val_col=None, group_col=None, welch=True, sort=False):
 
     Parameters
     ----------
-    a : array_like or pandas DataFrame object
+    a : Union[list, np.ndarray, DataFrame]
         An array, any object exposing the array interface or a pandas
         DataFrame.
 
@@ -1942,7 +2040,7 @@ def posthoc_tamhane(a, val_col=None, group_col=None, welch=True, sort=False):
 
     Returns
     -------
-    result : pandas DataFrame
+    result : pandas.DataFrame
         P values.
 
     Notes
@@ -1964,7 +2062,6 @@ def posthoc_tamhane(a, val_col=None, group_col=None, welch=True, sort=False):
     >>> x = x.melt(var_name='groups', value_name='values')
     >>> sp.posthoc_tamhane(x, val_col='values', group_col='groups')
     '''
-
     x, _val_col, _group_col = __convert_to_df(a, val_col, group_col)
     x = x.sort_values(by=[_group_col], ascending=True) if sort else x
 
@@ -2016,8 +2113,11 @@ def posthoc_tamhane(a, val_col=None, group_col=None, welch=True, sort=False):
     return DataFrame(vs, index=groups, columns=groups)
 
 
-def posthoc_tukey(a, val_col: str = None,
-                  group_col: str = None, sort: bool = False) -> DataFrame:
+def posthoc_tukey(
+        a: Union[list, np.ndarray, DataFrame],
+        val_col: str = None,
+        group_col: str = None,
+        sort: bool = False) -> DataFrame:
     '''Performs Tukey's all-pairs comparisons test for normally distributed data
     with equal group variances. For all-pairs comparisons in an
     one-factorial layout with normally distributed residuals and equal variances
@@ -2027,7 +2127,7 @@ def posthoc_tukey(a, val_col: str = None,
 
     Parameters
     ----------
-    a : array_like or pandas DataFrame object
+    a : Union[list, np.ndarray, DataFrame]
         An array, any object exposing the array interface or a pandas
         DataFrame.
 
@@ -2046,7 +2146,7 @@ def posthoc_tukey(a, val_col: str = None,
 
     Returns
     -------
-    result : pandas DataFrame
+    result : pandas.DataFrame
         P values.
 
     Notes
@@ -2067,7 +2167,6 @@ def posthoc_tukey(a, val_col: str = None,
     >>> x = x.melt(var_name='groups', value_name='values')
     >>> sp.posthoc_tukey(x, val_col='values', group_col='groups')
     '''
-
     x, _val_col, _group_col = __convert_to_df(a, val_col, group_col)
     x = x.sort_values(by=[_group_col], ascending=True) if sort else x
     groups = x[_group_col].unique()
@@ -2102,7 +2201,11 @@ def posthoc_tukey(a, val_col: str = None,
     return DataFrame(vs, index=groups, columns=groups)
 
 
-def posthoc_dscf(a, val_col=None, group_col=None, sort=False):
+def posthoc_dscf(
+        a: Union[list, np.ndarray, DataFrame],
+        val_col: str = None,
+        group_col: str = None,
+        sort: bool = False) -> DataFrame:
     '''Dwass, Steel, Critchlow and Fligner all-pairs comparison test for a
     one-factorial layout with non-normally distributed residuals. As opposed to
     the all-pairs comparison procedures that depend on Kruskal ranks, the DSCF
@@ -2111,7 +2214,7 @@ def posthoc_dscf(a, val_col=None, group_col=None, sort=False):
 
     Parameters
     ----------
-    a : array_like or pandas DataFrame object
+    a : Union[list, np.ndarray, DataFrame]
         An array, any object exposing the array interface or a pandas
         DataFrame.
 
@@ -2130,7 +2233,8 @@ def posthoc_dscf(a, val_col=None, group_col=None, sort=False):
 
     Returns
     -------
-    Pandas DataFrame containing p values.
+    result : pandas.DataFrame
+        P values.
 
     Notes
     -----
@@ -2157,7 +2261,6 @@ def posthoc_dscf(a, val_col=None, group_col=None, sort=False):
     >>> x = x.melt(var_name='groups', value_name='values')
     >>> sp.posthoc_dscf(x, val_col='values', group_col='groups')
     '''
-
     x, _val_col, _group_col = __convert_to_df(a, val_col, group_col)
     x = x.sort_values(by=[_group_col], ascending=True) if sort else x
     groups = x[_group_col].unique()
